"""Tests for ccx_messaging.utils.sliced_template module."""

import uuid

import pytest

from ccx_messaging.utils.sliced_template import SlicedTemplate


def test_standard_substitutions():
    """Check a standard substitution."""
    template = SlicedTemplate("$cluster_name")
    cluster_id = str(uuid.uuid4())

    assert template.safe_substitute(cluster_name=cluster_id) == cluster_id
    assert template.safe_substitute({"cluster_name": cluster_id}) == cluster_id
    assert template.safe_substitute({"other": "ignored"}, cluster_name=cluster_id) == cluster_id
    assert template.substitute(cluster_name=cluster_id) == cluster_id
    assert template.substitute({"cluster_name": cluster_id}) == cluster_id
    assert template.substitute({"other": "ignored"}, cluster_name=cluster_id) == cluster_id


def test_failed_substitutions():
    """Check some failing standard substitutions."""
    template = SlicedTemplate("$cluster_name")

    assert template.safe_substitute(other="ignored") == "$cluster_name"
    assert template.safe_substitute({"other": "ignored"}) == "$cluster_name"

    with pytest.raises(KeyError):
        template.substitute(other="ignored")
        template.substitute({"other": "ignored"})


def test_sliced_substitution():
    """Check a substitution with slicing."""
    template = SlicedTemplate("$cluster_name[0:20]")
    cluster_id = str(uuid.uuid4())

    expected = cluster_id[:20]

    assert template.safe_substitute(cluster_name=cluster_id) == expected
    assert template.safe_substitute({"cluster_name": cluster_id}) == expected
    assert template.substitute(cluster_name=cluster_id) == expected
    assert template.substitute({"cluster_name": cluster_id}) == expected


def test_failed_sliced_substitutions():
    """Check some failing standard substitutions."""
    template = SlicedTemplate("$cluster_name[0:20]")

    assert template.safe_substitute(other="ignored") == "$cluster_name[0:20]"
    assert template.safe_substitute({"other": "ignored"}) == "$cluster_name[0:20]"

    with pytest.raises(KeyError):
        template.substitute(other="ignored")
        template.substitute({"other": "ignored"})


def test_element_substitution():
    """Check a substitution with slicing."""
    template = SlicedTemplate("$cluster_name[15]")
    cluster_id = str(uuid.uuid4())

    expected = cluster_id[15]

    assert template.safe_substitute(cluster_name=cluster_id) == expected
    assert template.safe_substitute({"cluster_name": cluster_id}) == expected
    assert template.substitute(cluster_name=cluster_id) == expected
    assert template.substitute({"cluster_name": cluster_id}) == expected


def test_get_identifiers():
    """Check that get_identifiers find out real identifiers."""
    template = SlicedTemplate("$cluster_name[0:20]")
    assert template.get_identifiers() == ["cluster_name"]



"""Utilities related to Ingress message format."""

import json
import logging

import jsonschema

from ccx_messaging.error import CCXMessagingError
from ccx_messaging.schemas import ARCHIVE_SYNCED_SCHEMA, RULES_RESULTS_SCHEMA


LOG = logging.getLogger(__name__)


def parse_msg(message: bytes, schema: dict) -> dict:
    """Parse a bytes messages into a dictionary, decoding encoded values."""
    try:
        deserialized_message = json.loads(message)
        jsonschema.validate(instance=deserialized_message, schema=schema)

    except TypeError as ex:
        LOG.warning("Incorrect message type: %s", message)
        raise CCXMessagingError("Incorrect message type") from ex

    except json.JSONDecodeError as ex:
        LOG.warning("Unable to decode received message: %s", message)
        raise CCXMessagingError("Unable to decode received message") from ex

    except jsonschema.ValidationError as ex:
        LOG.warning("Invalid input message JSON schema: %s", deserialized_message)
        raise CCXMessagingError("Invalid input message JSON schema") from ex

    LOG.debug("JSON schema validated: %s", deserialized_message)
    return deserialized_message


def parse_archive_sync_msg(message: bytes) -> dict:
    """Parse archive sync message into dictionary."""
    return parse_msg(message, ARCHIVE_SYNCED_SCHEMA)


def parse_rules_results_msg(message: bytes) -> dict:
    """Parse rules results message into dictionary."""
    return parse_msg(message, RULES_RESULTS_SCHEMA)



# Copyright 2025 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module for testing `RulesResultsConsumer` class."""

import json
import os
import pytest
from unittest.mock import MagicMock, patch

from confluent_kafka import KafkaException
from insights.core.exceptions import InvalidContentType

from ccx_messaging.consumers.rules_results_consumer import RulesResultsConsumer
from ccx_messaging.error import CCXMessagingError


from . import KafkaMessage


_VALID_MESSAGE_CONTENT = {
    "path": "archives/compressed/aa/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/202101/20/031044.tar.gz",
    "metadata": {
        "cluster_id": "aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee",
    },
    "report": {
        "system": {},
        "reports": [],
        "skips": [],
        "pass": [],
        "info": [],
    },
}

_INVALID_TYPE_VALUES = [
    None,
    42,
    3.14,
    True,
    [],
    {},
]


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_get_url():
    """Test that `get_url` method retrieves path from correct field."""
    consumer = RulesResultsConsumer(None, None, None, incoming_topic=None)
    msg = {"file_path": "test"}
    assert consumer.get_url(msg) == "test"


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_get_url_invalid():
    """Test that `get_url` method raises exception if `file_path` field is missing."""
    consumer = RulesResultsConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(Exception):
        consumer.get_url({})


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_valid_msg():
    """Test that valid message is correctly deserialized."""
    consumer = RulesResultsConsumer(None, None, None, incoming_topic=None)
    msg = consumer.deserialize(KafkaMessage(json.dumps(_VALID_MESSAGE_CONTENT)))
    assert msg == _VALID_MESSAGE_CONTENT


@pytest.mark.parametrize("value", _INVALID_TYPE_VALUES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer")
def test_deserialize_invalid_type(mock_consumer, value):
    """Test that passing invalid data type to `deserialize` raises an exception."""
    sut = RulesResultsConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(value)


@pytest.mark.parametrize("value", _INVALID_TYPE_VALUES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer")
def test_deserialize_invalid_type_as_kafka_message(mock_consumer, value):
    """Test that passing invalid data type to `deserialize` raises an exception."""
    sut = RulesResultsConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(KafkaMessage(value))


_INVALID_MESSAGES = [
    {
        "metadata": {
            "cluster_id": "aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee",
        },
        "report": {
            "system": {},
            "reports": [],
            "skips": [],
            "pass": [],
            "info": [],
        },
    },
    {
        "path": "archives/compressed/aa/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/202101/20/031044.tar.gz",  # noqa: E501
        "report": {
            "system": {},
            "reports": [],
            "skips": [],
            "pass": [],
            "info": [],
        },
    },
    {
        "path": "archives/compressed/aa/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/202101/20/031044.tar.gz",  # noqa: E501
        "metadata": {},
        "report": {
            "system": {},
            "reports": [],
            "skips": [],
            "pass": [],
            "info": [],
        },
    },
    {
        "path": "archives/compressed/aa/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/202101/20/031044.tar.gz",  # noqa: E501
        "metadata": {
            "cluster_id": "aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee",
        },
    },
]


@pytest.mark.parametrize("value", _INVALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_invalid_msg_format(value):
    """Test that error is raised when deserializing messages with invalid format."""
    sut = RulesResultsConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(KafkaMessage(json.dumps(value)))


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_process_msg_temp_file():
    """Test that the temporary file is created and removed during the message processing."""
    file_name = ""

    def test_file_exists(_, msg):
        nonlocal file_name
        assert "file_path" in msg
        file_name = msg["file_path"]
        assert os.path.exists(file_name)

    consumer = RulesResultsConsumer(None, None, None, incoming_topic=None)
    with patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.process", test_file_exists):
        consumer.process_msg(KafkaMessage(json.dumps(_VALID_MESSAGE_CONTENT)))
    assert not os.path.exists(file_name)


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_process_msg_raise_error():
    """Test when the `process` message raises an error."""
    consumer = RulesResultsConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(KafkaException):
        consumer.process_msg(KafkaMessage(error=True))


@pytest.mark.parametrize("value", _INVALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_process_msg_invalid_format(value):
    """Test when the `process` message raises an error."""
    consumer = RulesResultsConsumer(None, None, None, incoming_topic=None)
    with patch(
        "ccx_messaging.consumers.rules_results_consumer.RulesResultsConsumer.process_dead_letter",
    ) as dlq_mock:
        consumer.process_msg(KafkaMessage(json.dumps(value)))
        assert dlq_mock.called


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_create_broker():
    """Test that broker contains all expected fields."""
    consumer = RulesResultsConsumer(None, None, None, incoming_topic=None)
    with patch(
        "ccx_messaging.consumers.rules_results_consumer.RulesResultsConsumer.create_report_path",
    ) as create_path_mock:
        create_path_mock.return_value = "mock_path"
        broker = consumer.create_broker(_VALID_MESSAGE_CONTENT)
        assert create_path_mock.called
        assert "report_path" in broker.keys()
        assert "cluster_id" in broker.keys()
        assert broker["report_path"] == "mock_path"
        assert broker["cluster_id"] == _VALID_MESSAGE_CONTENT["metadata"]["cluster_id"]


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_create_report_path():
    """Test that the report path is correctly created."""
    consumer = RulesResultsConsumer(None, None, None, incoming_topic=None)
    path = consumer.create_report_path(_VALID_MESSAGE_CONTENT)
    assert path == "insights/aa/aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee/202101/20/031044/insights.json"


_EXPECTED_EXCEPTIONS = [
    InvalidContentType(content_type="application/json"),
    CCXMessagingError("Test exception"),
    TimeoutError("Test timeout"),
    Exception("Test exception"),
]


@pytest.mark.parametrize("exception", _EXPECTED_EXCEPTIONS)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
@patch("ccx_messaging.consumers.rules_results_consumer.RulesResultsConsumer.process")
def test_process_to_dlq(process_mock, exception):
    """Test that the message is processed to DLQ when an error is raised."""
    process_mock.side_effect = exception
    sut = RulesResultsConsumer(None, None, None, incoming_topic=None)

    with patch(
        "ccx_messaging.consumers.rules_results_consumer.RulesResultsConsumer.process_dead_letter",
    ) as dlq_mock:
        sut.process_msg(KafkaMessage(json.dumps(_VALID_MESSAGE_CONTENT)))
        assert dlq_mock.called



# Copyright 2023 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing unit tests for the `KafkaConsumer` class."""

import datetime
import logging
import io
import time
from unittest.mock import MagicMock, patch

import pytest
from confluent_kafka import KafkaException
from freezegun import freeze_time

from ccx_messaging.consumers.kafka_consumer import KafkaConsumer
from ccx_messaging.error import CCXMessagingError

from . import KafkaMessage


# _REGEX_BAD_SCHEMA = r"^Unable to extract URL from input message: "
_INVALID_TYPE_VALUES = [
    None,
    42,
    3.14,
    True,
    [],
    {},
]


@pytest.mark.parametrize("value", _INVALID_TYPE_VALUES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer")
def test_deserialize_invalid_type(mock_consumer, value):
    """Test that passing invalid data type to `deserialize` raises an exception."""
    sut = KafkaConsumer(None, None, None, None)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(value)


_INVALID_MESSAGES = [
    "",
    "{}",
    '{"noturl":"https://s3.com/hash"}',
    '{"url":"value"',
    '"url":"value"}',
    '"url":"value"',
    '"{"url":"value"}"',
    # incorrect content of b64_identity (org_id missing)
    '{"url": "https://s3.com/hash", "b64_identity": "eyJpZGVudGl0eSI6IHsiaW50ZXJuYWwiOiB7Im1pc'
    '3Npbmdfb3JnX2lkIjogIjEyMzQ1Njc4In19fQo=", "timestamp": "2020-01-23T16:15:59.478901889Z"}',
    # incorrect format of base64 encoding
    '{"url": "https://s3.com/hash", "b64_identity": "eyJpZGVudGl0eSI6IHsiaW50ZXJuYWwiOiB7Im9yZ1'
    '9pZCI6ICIxMjM0NTY3OCJ9Cg=", "timestamp": "2020-01-23T16:15:59.478901889Z"}',
    # org_id not string
    '{"url": "https://s3.com/hash", "b64_identity": "eyJpZGVudGl0eSI6IHsKICAgICJhY2NvdW50X251bW'
    "JlciI6ICI5ODc2NTQzIiwKICAgICJhdXRoX3R5cGUiOiAiYmFzaWMtYXV0aCIsCiAgICAiaW50ZXJuYWwiOiB7CiAg"
    "ICAgICAgImF1dGhfdGltZSI6IDE0MDAsCiAgICAgICAgIm9yZ19pZCI6IDEyMzQ1Njc4CiAgICB9LAogICAgInR5cG"
    "UiOiAiVXNlciIsCiAgICAidXNlciI6IHsKICAgICAgICAiZW1haWwiOiAiam9obi5kb2VAcmVkaGF0LmNvbSIsCiAg"
    "ICAgICAgImZpcnN0X25hbWUiOiAiSW5zaWdodHMiLAogICAgICAgICJpc19hY3RpdmUiOiB0cnVlLAogICAgICAgIC"
    "Jpc19pbnRlcm5hbCI6IGZhbHNlLAogICAgICAgICJpc19vcmdfYWRtaW4iOiB0cnVlLAogICAgICAgICJsYXN0X25h"
    "bWUiOiAiUUUiLAogICAgICAgICJsb2NhbGUiOiAiZW5fVVMiLAogICAgICAgICJ1c2VybmFtZSI6ICJpbnNpZ2h0cy"
    '1tYXN0ZXIiCiAgICB9Cn0KfQo=", "timestamp": "2020-01-23T16:15:59.478901889Z"}',
]


@pytest.mark.parametrize("msg", _INVALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer")
def test_deserialize_invalid_format_str(mock_consumer, msg):
    """Test that passing a malformed message to `deserialize` raises an exception."""
    sut = KafkaConsumer(None, None, None, None)
    message = KafkaMessage(msg)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(message)


_VALID_MESSAGES = [
    (
        '{"url": "",'
        '"b64_identity": "eyJpZGVudGl0eSI6IHsiaW50ZXJuYWwiOiB7Im9yZ19pZCI6ICIxMjM0NTY3OCJ9fX0K",'
        '"timestamp": "2020-01-23T16:15:59.478901889Z"}',
        {
            "url": "",
            "identity": {"identity": {"internal": {"org_id": "12345678"}}},
            "timestamp": "2020-01-23T16:15:59.478901889Z",
            "cluster_name": None,
        },
    ),
    (
        '{"url": "https://s3.com/hash", "unused-property": null, '
        '"b64_identity": "eyJpZGVudGl0eSI6IHsiaW50ZXJuYWwiOiB7Im9yZ19pZCI6ICIxMjM0NTY3OCJ9fX0K",'
        '"timestamp": "2020-01-23T16:15:59.478901889Z"}',
        {
            "url": "https://s3.com/hash",
            "unused-property": None,
            "identity": {"identity": {"internal": {"org_id": "12345678"}}},
            "timestamp": "2020-01-23T16:15:59.478901889Z",
            "cluster_name": None,
        },
    ),
    (
        '{"account":12345678, "url":"any/url", '
        '"b64_identity": "eyJpZGVudGl0eSI6IHsiaW50ZXJuYWwiOiB7Im9yZ19pZCI6ICIxMjM0NTY3OCJ9fX0K",'
        '"timestamp": "2020-01-23T16:15:59.478901889Z"}',
        {
            "account": 12345678,
            "url": "any/url",
            "identity": {"identity": {"internal": {"org_id": "12345678"}}},
            "timestamp": "2020-01-23T16:15:59.478901889Z",
            "cluster_name": None,
        },
    ),
]

_MESSAGE_WITHOUT_IDENTITY = {
    "account": "12345678",
    "url": "any/url",
    "timestamp": "2020-01-23T16:15:59.478901889Z",
    "cluster_name": None,
}


@pytest.mark.parametrize("msg,value", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_valid_str(msg, value):
    """Test that proper string JSON input messages are correctly deserialized."""
    sut = KafkaConsumer(None, None, None, None)
    assert sut.deserialize(KafkaMessage(msg)) == value


@pytest.mark.parametrize("msg,value", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_valid_bytes(msg, value):
    """Test that proper string JSON input messages are correctly deserialized."""
    sut = KafkaConsumer(None, None, None, None)
    assert sut.deserialize(KafkaMessage(msg.encode())) == value


@pytest.mark.parametrize("msg,value", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_valid_bytearray(msg, value):
    """Test that proper string JSON input messages are correctly deserialized."""
    sut = KafkaConsumer(None, None, None, None)
    assert sut.deserialize(KafkaMessage(bytearray(msg.encode()))) == value


PLATFORM_SERVICE = "some_service"
_NO_HANDLE_HEADERS = [
    None,
    {},
    {"not_expected_key": b"ignored_value"},
    {"service": b"this is not my service"},
]

_HANDLE_HEADER = {"service": b"some_service"}


@pytest.mark.parametrize("headers", _NO_HANDLE_HEADERS)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_platform_filter(headers):
    """Test that filter by platform behaves as expected."""
    sut = KafkaConsumer(None, None, None, None, platform_service=PLATFORM_SERVICE)
    assert not sut.handles(KafkaMessage("{}", headers))


@pytest.mark.parametrize("headers", _NO_HANDLE_HEADERS + [_HANDLE_HEADER])
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_platform_filter_no_platform_service(headers):
    """Test that filter by platform behaves as expected."""
    sut = KafkaConsumer(None, None, None, None)
    assert sut.handles(KafkaMessage("{}", headers))


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_platform_filter_platform_service():
    """Test that filter by platform behaves as expected."""
    sut = KafkaConsumer(None, None, None, None, platform_service=PLATFORM_SERVICE)
    assert sut.handles(KafkaMessage("{}", _HANDLE_HEADER))


# This would have been a valid input, but it's supposed to be a `dict`, not `str`.
_DICT_STR = '{"url": "bucket/file"}'

_INVALID_RECORD_VALUES = [
    "",
    _DICT_STR,
    _DICT_STR.encode("utf-8"),
    bytearray(_DICT_STR.encode("utf-8")),
    [],
    {},
    {"noturl": "bucket/file"},
]

_VALID_RECORD_VALUES = [
    {"url": ""},
    {"url": "bucket/file"},
    {"url": "https://a-valid-domain.com/precious_url"},
]


@pytest.mark.parametrize("value", _INVALID_RECORD_VALUES)
def test_get_url_invalid(value):
    """Test that `KafkaConsumer.get_url` raises the appropriate exception."""
    with pytest.raises(CCXMessagingError):
        KafkaConsumer.get_url(None, value)


@pytest.mark.parametrize("value", _VALID_RECORD_VALUES)
def test_get_url_valid(value):
    """Test that `Consumer.get_url` returns the expected value."""
    assert KafkaConsumer.get_url(None, value) == value["url"]


_VALID_TOPICS = ["topic", "funny-topic"]
_VALID_GROUPS = ["group", "good-boys"]
_VALID_SERVERS = ["server", "great.server.net"]


@pytest.mark.parametrize("topic", _VALID_TOPICS)
@pytest.mark.parametrize("group", _VALID_GROUPS)
@pytest.mark.parametrize("server", _VALID_SERVERS)
def test_consumer_init_direct(topic, group, server):
    """Test of our Consumer constructor, using direct configuration options."""
    with patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer") as mock_consumer_init:
        with patch("os.environ", new={}):
            kwargs = {
                "group.id": group,
                "bootstrap.servers": server,
            }
            KafkaConsumer(None, None, None, topic, **kwargs)
            config = {
                "bootstrap.servers": server,
                "group.id": group,
            }
            mock_consumer_init.assert_called_with(config)


MAX_ELAPSED_TIME_BETWEEN_MESSAGES_TEST = 2


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
@patch(
    "ccx_messaging.consumers.kafka_consumer.MAX_ELAPSED_TIME_BETWEEN_MESSAGES",
    MAX_ELAPSED_TIME_BETWEEN_MESSAGES_TEST,
)
def test_elapsed_time_thread_no_warning_when_message_received():
    """Test elapsed time thread if new message received on time.

    Test that no warnings are sent if a new message is received before
    the defined MAX_ELAPSED_TIME_BETWEEN_MESSAGES.
    """
    buf = io.StringIO()
    log_handler = logging.StreamHandler(buf)

    logger = logging.getLogger()
    logger.level = logging.DEBUG
    logger.addHandler(log_handler)

    with patch("ccx_messaging.consumers.kafka_consumer.LOG", logger):
        sut = KafkaConsumer(None, None, None, None)
        assert sut.check_elapsed_time_thread
        buf.truncate(0)  # Empty buffer to make sure this test does what it should do
        sut.last_received_message_time = time.time()
        assert "No new messages in the queue since " not in buf.getvalue()
        time.sleep(MAX_ELAPSED_TIME_BETWEEN_MESSAGES_TEST - 1)
        sut.last_received_message_time = time.time()
        assert "No new messages in the queue since " not in buf.getvalue()

    logger.removeHandler(log_handler)


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
@patch(
    "ccx_messaging.consumers.kafka_consumer.MAX_ELAPSED_TIME_BETWEEN_MESSAGES",
    MAX_ELAPSED_TIME_BETWEEN_MESSAGES_TEST,
)
def test_elapsed_time_thread_warning_when_no_message_received():
    """Test elapsed time thread if no new message received on time.

    Test that warnings are sent if no new messages are received before
    the defined MAX_ELAPSED_TIME_BETWEEN_MESSAGES.
    """
    buf = io.StringIO()
    log_handler = logging.StreamHandler(buf)

    logger = logging.getLogger()
    logger.level = logging.DEBUG
    logger.addHandler(log_handler)

    with patch("ccx_messaging.consumers.kafka_consumer.LOG", logger):
        sut = KafkaConsumer(
            None, None, None, "topic", group_id="group", bootstrap_servers=["server"]
        )
        assert sut.check_elapsed_time_thread
        alert_message = "No new messages in the queue"
        # Make sure the thread woke up at least once
        time.sleep(2 * MAX_ELAPSED_TIME_BETWEEN_MESSAGES_TEST)
        assert alert_message in buf.getvalue()

    logger.removeHandler(log_handler)


@patch(
    "ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *args, **kwargs: MagicMock()
)
def test_handles_old_message():
    """Check that too old message is discarded."""
    msg = KafkaMessage(b"", None, 0)
    sut = KafkaConsumer(
        None,
        None,
        None,
        "topic",
        max_record_age=100,
        group_id="group",
        bootstrap_servers=["server"],
    )

    assert not sut.handles(msg)

    msg = KafkaMessage(b"", None, time.time() * 1000)  # using ms
    assert sut.handles(msg)


@patch(
    "ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *args, **kwargs: MagicMock()
)
def test_handles_disable_age_message_filtering():
    """Check that too old message is discarded."""
    msg = KafkaMessage(b"", None, 0)
    sut = KafkaConsumer(
        None,
        None,
        None,
        "topic",
        max_record_age=-1,
        group_id="group",
        bootstrap_servers=["server"],
    )

    assert sut.handles(msg)


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_process_msg_not_handled():
    """Test the `process_msg` method in the `KafkaConsumer` class.

    This method will check that neither sut.deserialize or sut.process are called
    """
    sut = KafkaConsumer(None, None, None, None)
    sut.deserialize = MagicMock()
    sut.process = MagicMock()
    sut.handles = lambda *a: False

    sut.process_msg(None)
    assert not sut.deserialize.called
    assert not sut.process.called

    sut.process_msg(KafkaMessage("any message that won't be handled"))
    assert not sut.deserialize.called
    assert not sut.process.called


@pytest.mark.parametrize("value,expected", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
@patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.handles", lambda *a, **k: True)
@patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.fire", lambda *a, **k: None)
def test_process_msg_handled(value, expected):
    """Check if `process_msg` behaves as expected."""
    sut = KafkaConsumer(None, None, None, None)
    input_msg = KafkaMessage(value)

    with patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.process") as process_mock:
        sut.process_msg(input_msg)
        process_mock.assert_called_with(expected)


@pytest.mark.parametrize("value,expected", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
@patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.handles", lambda *a, **k: True)
@patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.fire", lambda *a, **k: None)
def test_non_processed_to_dlq(value, expected):
    """Check that, if in some point an exception is raised, DLQ will handle it."""
    sut = KafkaConsumer(None, None, None, None)
    input_msg = KafkaMessage(value)

    with patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.process") as process_mock:
        with patch(
            "ccx_messaging.consumers.kafka_consumer.KafkaConsumer.process_dead_letter"
        ) as process_dlq_mock:
            process_mock.side_effect = [CCXMessagingError, TimeoutError, IndexError]
            sut.process_msg(input_msg)
            process_dlq_mock.assert_called_with(input_msg)

            sut.process_msg(input_msg)
            process_dlq_mock.assert_called_with(input_msg)

            sut.process_msg(input_msg)
            process_dlq_mock.assert_called_with(input_msg)


@pytest.mark.parametrize("value,expected", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_process_dead_letter_no_configured(value, expected):
    """Check that process_dead_letter method works as expected."""
    sut = KafkaConsumer(None, None, None, None)
    sut.dlq_producer = MagicMock()  # inject a mock producer
    sut.dlq_producer.__bool__.return_value = False

    input_message = KafkaMessage(value)

    sut.process_dead_letter(input_message)
    assert not sut.dlq_producer.produce.called


@pytest.mark.parametrize("value,expected", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
@patch("ccx_messaging.consumers.kafka_consumer.Producer")
def test_process_dead_letter_message(producer_init_mock, value, expected):
    """Check behaviour when DLQ is properly configured."""
    dlq_topic_name = "dlq_topic"
    producer_mock = MagicMock()
    producer_init_mock.return_value = producer_mock

    sut = KafkaConsumer(None, None, None, None, dead_letter_queue_topic=dlq_topic_name)
    assert producer_init_mock.called

    message_mock = MagicMock()
    message_mock.value.return_value = value

    sut.process_dead_letter(message_mock)
    producer_mock.produce.assert_called_with(dlq_topic_name, value)


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer")
def test_run_success(consumer_init_mock):
    """Check the run method process the message as many times as needed."""
    consumer_mock = MagicMock()
    consumer_init_mock.return_value = consumer_mock

    consumer_mock.consume.side_effect = [
        [KafkaMessage(b"0")],
        [KafkaMessage(b"1"), KafkaMessage(b"2")],
        KeyboardInterrupt(),
    ]

    sut = KafkaConsumer(None, None, None, "topic")
    sut.process_msg = MagicMock()
    sut.run()

    assert consumer_mock.consume.call_count == 3
    assert consumer_mock.close.call_count == 1
    assert sut.process_msg.call_count == 3


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer")
def test_run_fail(consumer_init_mock):
    """Check the run method process the message as many times as needed."""
    consumer_mock = MagicMock()
    consumer_init_mock.return_value = consumer_mock

    consumer_mock.consume.side_effect = [
        KafkaException(),
    ]

    sut = KafkaConsumer(None, None, None, "topic")
    sut.process_msg = MagicMock()
    sut.run()

    assert consumer_mock.consume.call_count == 1
    assert consumer_mock.close.call_count == 1
    assert not sut.process_msg.called


@pytest.mark.parametrize(
    "deserialized_msg", [msg[1] for msg in _VALID_MESSAGES] + [_MESSAGE_WITHOUT_IDENTITY]
)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_broker_creation(deserialized_msg):
    """Check that create_broker returns the expected values."""
    sut = KafkaConsumer(None, None, None, "topic")
    broker = sut.create_broker(deserialized_msg)
    msg_timestamp = datetime.datetime.fromisoformat(deserialized_msg["timestamp"])

    assert broker["original_path"] == deserialized_msg["url"]
    identity = (
        deserialized_msg.get("identity", {})
        .get("identity", {})
        .get("internal", {})
        .get("org_id", None)
    )
    assert broker["org_id"] == identity
    assert broker["year"] == f"{msg_timestamp.year:04}"
    assert broker["month"] == f"{msg_timestamp.month:02}"
    assert broker["day"] == f"{msg_timestamp.day:02}"
    assert broker["hour"] == f"{msg_timestamp.hour:02}"
    assert broker["minute"] == f"{msg_timestamp.minute:02}"
    assert broker["second"] == f"{msg_timestamp.second:02}"
    assert broker["time"] == (
        f"{msg_timestamp.hour:02}{msg_timestamp.minute:02}{msg_timestamp.second:02}"
    )


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
@patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.handles", lambda *a, **k: True)
@patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.fire", lambda *a, **k: None)
def test_last_received_message_time_is_updated():
    """Check that the variable last_received_message_time used by a Thread is correctly updated.

    [CCXDEV-14812] the variable is never updated
    """
    t1 = datetime.datetime(2025, 1, 31, 12, 0, 0, tzinfo=datetime.timezone.utc)
    t2 = datetime.datetime(2025, 1, 31, 12, 20, 0, tzinfo=datetime.timezone.utc)

    with freeze_time(t1) as frozen_time:
        sut = KafkaConsumer(None, None, None, None)
        input_msg = KafkaMessage("{}")

        frozen_time.move_to(t2)

        with patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.process", lambda: None):
            sut.process_msg(input_msg)

        assert sut.last_received_message_time == t2.timestamp()



# Copyright 2023 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Submodule to configure logging stuff that cannot be afford from configuration file."""

import logging
import os
import platform
import uuid

from pythonjsonlogger import json as jsonlogger
from boto3.session import Session
from watchtower import CloudWatchLogHandler


def setup_watchtower(logging_config=None):
    """Setups the CloudWatch handler if the proper configuration is provided."""
    enabled = os.getenv("LOGGING_TO_CW_ENABLED", "False").lower() in ("true", "1", "t", "yes")
    if not enabled:
        return

    aws_config_vars = (
        "CW_AWS_ACCESS_KEY_ID",
        "CW_AWS_SECRET_ACCESS_KEY",
        "AWS_REGION_NAME",
        "CW_LOG_GROUP",
        "CW_STREAM_NAME",
    )

    if any(os.environ.get(key, "").strip() == "" for key in aws_config_vars):
        return

    # Get the log level from CW_LOG_LEVEL. If not, default to INFO
    log_level = logging.getLevelName(os.getenv("CW_LOG_LEVEL", "INFO"))

    session = Session(
        aws_access_key_id=os.environ["CW_AWS_ACCESS_KEY_ID"],
        aws_secret_access_key=os.environ["CW_AWS_SECRET_ACCESS_KEY"],
        region_name=os.environ["AWS_REGION_NAME"],
    )
    client = session.client("logs")

    root_logger = logging.getLogger()

    handler = CloudWatchLogHandler(
        boto3_client=client,
        log_group_name=os.environ["CW_LOG_GROUP"],
        log_stream_name=os.environ["CW_STREAM_NAME"],
        create_log_group=False,
    )

    if logging_config is not None:
        log_format = logging_config.get("formatters", {}).get("cloudwatch", {}).get("format")
        handler.setFormatter(CloudWatchFormatter(log_format))

    try:
        handler.setLevel(log_level)

    except ValueError:
        root_logger.warning(
            "Log level for cloudwatch cannot be set to %s. Default to INFO", log_level
        )
        handler.setLevel(logging.INFO)

    root_logger.addHandler(handler)


class CloudWatchFormatter(jsonlogger.JsonFormatter):
    """Class that implements formatter for logging to CloudWatch."""

    def __init__(self, *args, **kwargs):
        """Initialize CloudWatchFormatter."""
        super().__init__(*args, **kwargs)

        self.hostname = platform.node()
        self.mac_address = get_mac_address()

    def format(self, record):
        """Format the record."""
        record.mac_address = self.mac_address
        record.hostname = self.hostname
        return super().format(record)


def get_mac_address():
    """Get mac address or None if it is not possible."""
    mac_address = uuid.getnode()

    # Ignore if it wasn't successful, see help of uuid.getnode():

    # If all attempts to obtain the hardware address fail, we
    # choose a random 48-bit number with its eighth bit set to 1 as recommended
    # in RFC 4122.

    # By 8th they meant 40th. From sources of getnode:
    # def _random_getnode():
    #     """Get a random node ID."""
    #     # RFC 4122, $4.1.6 says "For systems with no IEEE address, a randomly or
    #     # pseudo-randomly generated value may be used; see Section 4.5.  The
    #     # multicast bit must be set in such addresses, in order that they will
    #     # never conflict with addresses obtained from network cards."
    #     #
    #     # The "multicast bit" of a MAC address is defined to be "the least
    #     # significant bit of the first octet".  This works out to be the 41st bit
    #     # counting from 1 being the least significant bit, or 1<<40.
    #     #
    #     # See https://en.wikipedia.org/wiki/MAC_address#Unicast_vs._multicast
    #     import random
    #     return random.getrandbits(48) | (1 << 40)
    if (mac_address >> 40) & 1:
        # if this bit is 1, ignore, it's better than have bunch of random numbers in logs
        mac_address = None

    # finally format it to the human readable format
    if mac_address is not None:
        mac_address = ":".join(reversed([hex((mac_address >> i * 8) & 0xFF)[2:] for i in range(6)]))

    return mac_address



"""Tests for the utility class SentryMonitoredBroker."""

import time
from uuid import uuid4
from unittest.mock import patch

import pytest
from insights.core.dr import MissingRequirements
from insights.core.spec_factory import ContentException


from ccx_messaging.monitored_broker import SentryMonitoredBroker


def test_usage():
    """Check that normal usage for the broker is kept across inheritance."""
    broker = SentryMonitoredBroker()
    cluster_id = uuid4()
    timestamp = time.time()

    broker["cluster_id"] = cluster_id
    broker["timestamp"] = timestamp

    assert broker["cluster_id"] == cluster_id
    assert broker["timestamp"] == timestamp


@patch("ccx_messaging.monitored_broker.capture_exception")
def test_add_exception(sentry_capture_exception_mock):
    """Check add_exception behavior."""
    broker = SentryMonitoredBroker()

    broker.add_exception(None, KeyboardInterrupt, None)
    assert sentry_capture_exception_mock.called


NO_CAPTURE_EXCEPTIONS = [MissingRequirements, ContentException]


@pytest.mark.parametrize("exception", NO_CAPTURE_EXCEPTIONS)
def test_add_exception_no_capture(exception):
    """Check that selected exceptions are not captured by Sentry."""
    broker = SentryMonitoredBroker()

    with patch("ccx_messaging.monitored_broker.capture_exception") as capture_exception_mock:
        # Adding an argument to constructor because it is required by some of the exceptions
        broker.add_exception(None, exception("some message"), None)
        assert not capture_exception_mock.called



"""Module containing tests for the consumers."""

from confluent_kafka import TIMESTAMP_NOT_AVAILABLE


class KafkaMessage:
    """Test double for the confluent_kafka.Message class."""

    def __init__(self, msg=None, headers=None, timestamp=None, error=False):
        """Initialize a KafkaMessage test double."""
        self.msg = msg
        self._headers = headers
        self._timestamp = timestamp
        self.topic = lambda: "topic"
        self.partition = lambda: 0
        self.offset = lambda: 0
        self.value = lambda: self.msg
        if error:
            self.error = lambda: "error"
        else:
            self.error = lambda: None

        self.headers = lambda: self._headers

    def timestamp(self):
        """Test double for the Message.timestamp function."""
        if self._timestamp is None:
            return TIMESTAMP_NOT_AVAILABLE, None

        else:
            return None, self._timestamp



# Copyright 2023 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Test for the ccx_messaging.utils.kafka_config module."""

from ccx_messaging.utils.kafka_config import (
    translate_kafka_configuration,
    kafka_producer_config_cleanup,
)


def test_translate_kafka_configuration():
    """Check the translation of a default Kafka config to kafka-python configuration."""
    kafka_config = {
        "bootstrap.servers": "amazing-broker:9092",
        "ssl.ca.location": "/tmp/awesome.crt",
        "sasl.mechanisms": "PLAIN",
        "sasl.username": "incredible_user",
        "sasl.password": "supersecret_password",
        "security.protocol": "sasl_ssl",
    }

    expected_lib_config = {
        "bootstrap_servers": "amazing-broker:9092",
        "ssl_cafile": "/tmp/awesome.crt",
        "sasl_mechanism": "PLAIN",
        "sasl_plain_username": "incredible_user",
        "sasl_plain_password": "supersecret_password",
        "security_protocol": "sasl_ssl",
    }

    assert translate_kafka_configuration(kafka_config) == expected_lib_config


def test_translate_kafka_configuration_unexpected_keys_ignored():
    """Check the translation ignore unexpected broker configuration prameters."""
    kafka_config = {
        "retry_backoff_ms": 500,
        "should_ignore_key": "I shouldn't be there",
        "sasl.password": "supersecret_password",
        "security.protocol": "sasl_ssl",
    }

    expected_lib_config = {
        "sasl_plain_password": "supersecret_password",
        "security_protocol": "sasl_ssl",
    }

    assert translate_kafka_configuration(kafka_config) == expected_lib_config


def test_kafka_producer_config_cleanup():
    """Check the clean up function for producer' configurations."""
    kafka_consumer_config = {
        "group.id": "should delete",
        "bootstrap.servers": "kafka:9092",
        "session.timeout.ms": "1000",
        "heartbeat.interval.ms": "1000",
        "max.poll.interval.ms": "1000",
    }

    cfg = kafka_producer_config_cleanup(kafka_consumer_config)

    assert "group.id" not in cfg
    assert "session.timeout.ms" not in cfg
    assert "heartbeat.interval.ms" not in cfg
    assert "max.poll.interval.ms" not in cfg



# Copyright 2023 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module implementing a DVO Metrics publisher to Kafka topic."""

import json
import logging
from typing import Dict

from ccx_messaging.error import CCXMessagingError
from ccx_messaging.publishers.kafka_publisher import KafkaPublisher


log = logging.getLogger(__name__)


class DVOMetricsPublisher(KafkaPublisher):
    """DVOMetricsPublisher handles the result of the extraction of DVO metrics from an archive."""

    def publish(self, input_msg: Dict, report: str) -> None:
        """Publish an EOL-terminated JSON message to the output Kafka topic.

        The response is assumed to be a string representing a valid JSON object.
        A newline character will be appended to it, it will be converted into
        a byte array using UTF-8 encoding and the result of that will be sent
        to the producer to produce a message in the output Kafka topic.
        """
        output_msg = {}

        try:
            report = json.loads(report)
        except (TypeError, json.decoder.JSONDecodeError) as err:
            raise CCXMessagingError("Could not parse report; report is not in JSON format") from err

        if "workload_recommendations" not in report.keys():
            log.debug("Report does not contain DVO related results; skipping")
            return

        report.pop("reports", None)

        try:
            org_id = int(input_msg["identity"]["identity"]["internal"]["org_id"])
        except (ValueError, KeyError, TypeError) as err:
            log.warning("Error extracting the OrgID: %s", err)
            raise CCXMessagingError("Error extracting the OrgID") from err

        try:
            account_number = int(input_msg["identity"]["identity"]["account_number"])
        except (ValueError, KeyError, TypeError) as err:
            log.warning("Error extracting the Account number: %s", err)
            account_number = None

        if "cluster_name" not in input_msg:
            raise CCXMessagingError("Can't find 'cluster_name'")

        msg_version = report.pop("version", 0)
        output_msg = {
            "OrgID": org_id,
            "ClusterName": input_msg["cluster_name"],
            "Metrics": report,
            "RequestId": input_msg.get("request_id"),
            "LastChecked": input_msg.get("timestamp"),
            "Version": msg_version,
        }

        if account_number is not None:
            output_msg["AccountNumber"] = account_number

        message = json.dumps(output_msg) + "\n"

        log.debug("Sending response to the %s topic.", self.topic)
        # Convert message string into a byte array.
        self.produce(message.encode("utf-8"))
        log.debug("Message has been sent successfully.")
        log.debug(
            'Message context: OrgId=%s, AccountNumber=%s, ClusterName="%s"',
            output_msg["OrgID"],
            output_msg.get("AccountNumber", ""),
            output_msg["ClusterName"],
        )



# Copyright 2024 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may naot use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for the SyncedArchivePublisher class."""

import json
import gzip
from unittest.mock import MagicMock

import pytest
from ccx_messaging.publishers.synced_archive_publisher import SyncedArchivePublisher

BEST_COMPRESSION = 9

INPUT_MSG = [
    pytest.param(
        {
            "path": "target/path",
            "original_path": "original/path",
            "metadata": {"cluster_id": "12345", "external_organization": "54321"},
        }
    )
]


def timeStampMasking(message):
    """Mask four bytes in Gzip stream that contain timestamp."""
    message = list(message)
    message[4] = 0
    message[5] = 0
    message[6] = 0
    message[7] = 0
    message = bytes(message)
    return message


def test_init():
    """Check that init creates a valid object."""
    kakfa_config = {
        "bootstrap.servers": "kafka:9092",
    }
    SyncedArchivePublisher(outgoing_topic="topic name", **kakfa_config)


@pytest.mark.parametrize("input", INPUT_MSG)
def test_compressing_disabled(input):
    """Check if message is not gzipped if compression is disabled."""
    input = bytes(json.dumps(input) + "\n", "utf-8")
    expected_output = input
    kakfa_config = {"bootstrap.servers": "kafka:9092"}
    pub = SyncedArchivePublisher(outgoing_topic="topic-name", **kakfa_config)
    pub.producer = MagicMock()
    pub.produce(input)
    pub.producer.produce.assert_called_with("topic-name", expected_output)


@pytest.mark.parametrize("input", INPUT_MSG)
def test_compressing_enabled(input):
    """Check if message is gzipped if compression is enabled."""
    input = bytes(json.dumps(input) + "\n", "utf-8")
    expected_output = timeStampMasking(gzip.compress(input, compresslevel=BEST_COMPRESSION))
    kakfa_config = {"bootstrap.servers": "kafka:9092", "compression": "gzip"}
    pub = SyncedArchivePublisher(outgoing_topic="topic-name", **kakfa_config)
    pub.producer = MagicMock()
    pub.produce(input)
    outgoing_topic = pub.producer.produce.call_args[0][0]
    outgoing_message = timeStampMasking(pub.producer.produce.call_args[0][1])
    assert outgoing_message == expected_output and outgoing_topic == "topic-name"



# Copyright 2025 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for the RuleProcessingPublisher class."""

import json
from unittest.mock import MagicMock, patch

import pytest
from confluent_kafka import KafkaException

from ccx_messaging.error import CCXMessagingError
from ccx_messaging.publishers.idp_rule_processing_publisher import IDPRuleProcessingPublisher


def test_init():
    """Check that init creates a valid object."""
    kakfa_config = {
        "bootstrap.servers": "kafka:9092",
    }
    IDPRuleProcessingPublisher(outgoing_topic="topic name", **kakfa_config)


INVALID_TOPIC_NAMES = [
    None,
    b"Topic name",
    [],
    {},
    4,
    5.5,
    5 + 2j,
]


@pytest.mark.parametrize("topic_name", INVALID_TOPIC_NAMES)
def test_init_invalid_topic(topic_name):
    """Check what happens when the output_topic parameter is not valid."""
    with pytest.raises(CCXMessagingError):
        IDPRuleProcessingPublisher(topic_name)


INVALID_KWARGS = [
    {},
    {"bootstrap_servers": "kafka:9092"},
    {"bootstrap.servers": "kafka:9092", "unknown_option": "value"},
]


@pytest.mark.parametrize("kwargs", INVALID_KWARGS)
def test_bad_initialization(kwargs):
    """Check that init fails when using not valid kwargs."""
    with pytest.raises(KafkaException):
        IDPRuleProcessingPublisher(outgoing_topic="topic", **kwargs)


@pytest.mark.parametrize("kafka_broker_cfg", INVALID_KWARGS)
@pytest.mark.parametrize("kwargs", INVALID_KWARGS)
def test_bad_init_with_kafka_config(kafka_broker_cfg, kwargs):
    """Check that init fails when using not valid kwargs."""
    with pytest.raises(KafkaException):
        IDPRuleProcessingPublisher(outgoing_topic="topic", **kwargs)


INVALID_INPUT_MSGS = [
    None,
    "",
    1,
    2.0,
    3 + 1j,
    [],
    {},  # right type, missing path and metadata
    {"path": ""},  # missing metadata
    {"path": "", "metadata": {}},  # missing metadata-cluster_id
]


@pytest.mark.parametrize("wrong_input_msg", INVALID_INPUT_MSGS)
def test_publish_bad_argument(wrong_input_msg):
    """Check that invalid messages passed by the framework are handled gracefully."""
    sut = IDPRuleProcessingPublisher("outgoing_topic", {"bootstrap.servers": "kafka:9092"})
    sut.producer = MagicMock()

    with pytest.raises(CCXMessagingError):
        sut.publish(wrong_input_msg, {})
        assert not sut.producer.produce.called


VALID_INPUT_MSG = [
    pytest.param(
        {
            "path": "bucket/path/to/archive.tgz",
            "metadata": {
                "cluster_id": "uuid",
            },
            "request_id": "ingress-service-7777777777-q4444/4444444444-000001",
        },
        {
            "path": "bucket/path/to/archive.tgz",
            "metadata": {
                "cluster_id": "uuid",
            },
            "report": {
                "reports": [],
            },
            "request_id": "ingress-service-7777777777-q4444/4444444444-000001",
        },
        id="minimal valid",
    ),
    pytest.param(
        {
            "path": "bucket/path/to/archive.tgz",
            "original_path": "other_than_current_path",
            "metadata": {"cluster_id": "uuid", "external_organization": "an organization"},
            "request_id": "ingress-service-7777777777-q4444/4444444444-000001",
        },
        {
            "path": "bucket/path/to/archive.tgz",
            "metadata": {"cluster_id": "uuid", "external_organization": "an organization"},
            "report": {
                "reports": [],
            },
            "request_id": "ingress-service-7777777777-q4444/4444444444-000001",
        },
        id="adding optional elements",
    ),
]


@pytest.mark.parametrize("input, expected_output", VALID_INPUT_MSG)
def test_publish_valid(input, expected_output):
    """Check that Kafka producer is called with an expected message."""
    report = '{"reports": []}'

    expected_output = json.dumps(expected_output)
    sut = IDPRuleProcessingPublisher("outgoing_topic", {"bootstrap.servers": "kafka:9092"})
    sut.producer = MagicMock()

    sut.publish(input, report)
    sut.producer.produce.assert_called_with("outgoing_topic", expected_output.encode())


INVALID_REPORTS = [
    None,
    1,
    2.0,
    1 + 3j,
    [],
    {},
    "",
]


@pytest.mark.parametrize("invalid_report", INVALID_REPORTS)
def test_publish_invalid_report(invalid_report):
    """Check the behaviour of publish when an invalid report is received."""
    sut = IDPRuleProcessingPublisher("outgoing_topic", {"bootstrap.servers": "kafka:9092"})
    sut.producer = MagicMock()

    with pytest.raises(CCXMessagingError):
        sut.publish(VALID_INPUT_MSG, invalid_report)
        assert not sut.producer.produce.called


@pytest.mark.parametrize("input,output", VALID_INPUT_MSG)
def test_error(input, output):
    """Check that error just prints a log."""
    _ = output  # output values are not needed

    sut = IDPRuleProcessingPublisher("outgoing_topic", {"bootstrap.servers": "kafka:9092"})

    with patch("ccx_messaging.publishers.kafka_publisher.log") as log_mock:
        sut.error(input, None)
        assert log_mock.warning.called



# Copyright 2020, 2021, 2022 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing unit tests for the `HTTPDownloader` class."""

from unittest.mock import MagicMock, patch

import requests
import pytest

from ccx_messaging.downloaders.http_downloader import HTTPDownloader, parse_human_input
from ccx_messaging.error import CCXMessagingError


_INVALID_FILE_SIZES = [42, 2.71, True, [], {}, "1234J"]
_VALID_FILE_SIZES = ["1K", "2M", "3G", "4T", "5Ki", "6Mi", "7Gi", "8Ti"]


@pytest.mark.parametrize("file_size", _INVALID_FILE_SIZES)
def test_parse_human_input_invalid_file_sizes(file_size):
    """Test that passing invalid data to parse_human_input raise an exception."""
    with pytest.raises((TypeError, ValueError)):
        parse_human_input(file_size)


@pytest.mark.parametrize("file_size", _VALID_FILE_SIZES)
def test_parse_human_input_valid_file_sizes(file_size):
    """Test that passing valid data to parse_human_input doesn't raise an exception."""
    parse_human_input(file_size)


_REGEX_BAD_URL_FORMAT = r"^Invalid URL format"
_INVALID_TYPE_URLS = [42, 2.71, True, [], {}]


@pytest.mark.parametrize("url", _INVALID_TYPE_URLS)
def test_get_invalid_type(url):
    """Test that passing invalid data type to `get` raises an exception."""
    with pytest.raises(TypeError):
        sut = HTTPDownloader()
        with sut.get(url):
            pass


_INVALID_URLS = [None, "", "ftp://server", "bucket/file"]


@pytest.mark.parametrize("url", _INVALID_URLS)
def test_get_invalid_url(url):
    """Test that passing invalid URL to `get` raises an exception."""
    with pytest.raises(CCXMessagingError, match=_REGEX_BAD_URL_FORMAT):
        sut = HTTPDownloader()
        with sut.get(url):
            pass


_VALID_URLS = [
    "https://zzzzzzzzzzzzzzzzzzzzzzzz.s3.amazonaws.com/aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
    "?X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential="
    "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ&X-Amz-Date=19700101T000000Z"
    "&X-Amz-Expires=86400&X-Amz-SignedHeaders=host&X-Amz-Signature="
    "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
    "https://insights-dev-upload-perm.s3.amazonaws.com/upload-service-1-48nb7/gPRz2EdWpr-000144"
    "?X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=FillInCorrectDataHere%2F2C2D0311%2F"
    "us-east-1%2Fs3%2Faws4_request&X-Amz-Date=20200311T080728Z&X-Amz-Expires=86400&"
    "X-Amz-SignedHeaders=host&"
    "X-Amz-Signature=0000cafe0000babe0000cafe0000babe0000cafe0000babe0000cafe0000babe",
    "https://s3.us-east-1.amazonaws.com/insights-ingress-prod/aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
    "?X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential="
    "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ&X-Amz-Date=20201201T210535Z&"
    "X-Amz-Expires=86400&X-Amz-SignedHeaders=host&X-Amz-Signature="
    "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
]


@patch("requests.get")
@pytest.mark.parametrize("url", _VALID_URLS)
def test_get_valid_url(get_mock, url):
    """Test that passing a valid URL the `get` method tries to download it."""
    response_mock = MagicMock()
    get_mock.return_value = response_mock
    response_mock.content = b"file content"
    sut = HTTPDownloader()

    with sut.get(url) as filename:
        with open(filename, "rb") as file_desc:
            file_content = file_desc.read()
            assert file_content == response_mock.content


@patch("requests.get")
def test_get_empty_archive(get_mock):
    """Test that passing an empty archive to `get` raises an exception."""
    response_mock = MagicMock()
    get_mock.return_value = response_mock
    response_mock.content = b""

    with pytest.raises(CCXMessagingError, match="Empty input archive"):
        sut = HTTPDownloader(allow_unsafe_links=True)
        with sut.get("https://example.com/empty.tar.gz"):
            pass


@patch("requests.get")
def test_max_archive_size_enforced(get_mock):
    """Test that the max archive size is enforced."""
    response_mock = MagicMock()
    get_mock.return_value = response_mock
    response_mock.content = b"file content"

    with pytest.raises(CCXMessagingError, match="The archive is too big. Skipping"):
        sut = HTTPDownloader(max_archive_size="2", allow_unsafe_links=True)
        with sut.get("https://example.com/large.tar.gz"):
            pass


@patch("requests.get")
def test_connection_error(get_mock):
    """Test that a connection error raises an exception."""
    get_mock.side_effect = requests.exceptions.ConnectionError("Connection error")

    with pytest.raises(CCXMessagingError, match="Connection error while downloading the file"):
        sut = HTTPDownloader(allow_unsafe_links=True)
        with sut.get("https://example.com/connection_error.tar.gz"):
            pass


@patch("requests.get")
def test_other_error(get_mock):
    """Test that other errors raise an exception."""
    get_mock.side_effect = Exception("Other error")

    with pytest.raises(CCXMessagingError, match="Unknown error while downloading the file"):
        sut = HTTPDownloader(allow_unsafe_links=True)
        with sut.get("https://example.com/other_error.tar.gz"):
            pass


@patch("ccx_messaging.downloaders.http_downloader.LOG.warning")
@patch("requests.get")
def test_additional_data_handling(get_mock, log_mock):
    """Test that an invalid URL format raises an exception."""
    expected_additional_data = {"archive_path": "test.tar"}
    exception_to_be_raised = CCXMessagingError(
        "Test exception", additional_data=expected_additional_data
    )
    get_mock.side_effect = exception_to_be_raised

    with pytest.raises(CCXMessagingError):
        sut = HTTPDownloader(allow_unsafe_links=True)
        with sut.get("invalid_url"):
            pass

    log_mock.assert_called_with(
        "Error while downloading the file: %s",
        exception_to_be_raised,
        extra=expected_additional_data,
    )



# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Clowder integration functions."""

import logging

import yaml
import app_common_python
from app_common_python.types import BrokerConfigAuthtypeEnum


logger = logging.getLogger(__name__)


def apply_clowder_config(manifest):
    """Apply Clowder config values to ICM config manifest."""
    Loader = getattr(yaml, "CSafeLoader", yaml.SafeLoader)
    config = yaml.load(manifest, Loader=Loader)

    _add_buckets_config(config)
    _add_kafka_config(config)

    return config


# flake8: noqa: C901
def _add_kafka_config(config):
    # Find the Payload Tracker watcher, as it might be affected by config changes
    pt_watcher_name = "ccx_messaging.watchers.payload_tracker_watcher.PayloadTrackerWatcher"
    pt_watcher = None
    for watcher in config["service"]["watchers"]:
        if watcher["name"] == pt_watcher_name:
            pt_watcher = watcher
            break

    clowder_broker_config = app_common_python.LoadedConfig.kafka.brokers[0]
    kafka_urls = app_common_python.KafkaServers
    print("Kafka URLs:", kafka_urls)

    kafka_broker_config = {"bootstrap.servers": ",".join(kafka_urls)}

    if clowder_broker_config.cacert:
        # Current Kafka library is not able to handle the CA file, only a path to it
        # FIXME: Duplicating parameters in order to be used by both Kafka libraries
        ssl_ca_location = app_common_python.LoadedConfig.kafka_ca()
        kafka_broker_config["ssl.ca.location"] = ssl_ca_location

    if BrokerConfigAuthtypeEnum.valueAsString(clowder_broker_config.authtype) == "sasl":
        kafka_broker_config.update(
            {
                "sasl.mechanisms": clowder_broker_config.sasl.saslMechanism,
                "sasl.username": clowder_broker_config.sasl.username,
                "sasl.password": clowder_broker_config.sasl.password,
                "security.protocol": clowder_broker_config.sasl.securityProtocol,
            }
        )

    configure_consumer, configure_publisher = False, False
    if "kwargs" in config["service"]["consumer"]:
        print("Configuring consumer")
        configure_consumer = True
    if "kwargs" in config["service"]["publisher"]:
        print("Configuring publisher")
        configure_publisher = True

    if pt_watcher:
        pt_watcher["kwargs"]["kafka_broker_config"] = kafka_broker_config

    print("Kafka configuration updated from Clowder configuration")

    if configure_consumer:
        config["service"]["consumer"]["kwargs"]["kafka_broker_config"] = kafka_broker_config
        consumer_topic = config["service"]["consumer"]["kwargs"].get("incoming_topic")
        dlq_topic = config["service"]["consumer"]["kwargs"].get("dead_letter_queue_topic")
        if consumer_topic in app_common_python.KafkaTopics:
            topic_cfg = app_common_python.KafkaTopics[consumer_topic]
            config["service"]["consumer"]["kwargs"]["incoming_topic"] = topic_cfg.name
        else:
            print("The consumer topic cannot be found in Clowder mapping. It can cause errors")

        if dlq_topic in app_common_python.KafkaTopics:
            topic_cfg = app_common_python.KafkaTopics[dlq_topic]
            config["service"]["consumer"]["kwargs"]["dead_letter_queue_topic"] = topic_cfg.name

    if configure_publisher:
        config["service"]["publisher"]["kwargs"]["kafka_broker_config"] = kafka_broker_config
        producer_topic = config["service"]["publisher"]["kwargs"].get("outgoing_topic")
        if producer_topic in app_common_python.KafkaTopics:
            topic_cfg = app_common_python.KafkaTopics[producer_topic]
            config["service"]["publisher"]["kwargs"]["outgoing_topic"] = topic_cfg.name
        else:
            print("The publisher topic cannot be found in Clowder mapping. It can cause errors")

    payload_tracker_topic = pt_watcher["kwargs"].pop("topic") if pt_watcher else None

    if pt_watcher and payload_tracker_topic in app_common_python.KafkaTopics:
        topic_cfg = app_common_python.KafkaTopics[payload_tracker_topic]
        pt_watcher["kwargs"]["topic"] = topic_cfg.name
    else:
        print(
            "The Payload Tracker watcher topic cannot be found in Clowder mapping. "
            "It can cause errors",
        )


def _update_bucket_config(bucket_name, configuration):
    buckets = app_common_python.ObjectBuckets
    common_config = app_common_python.LoadedConfig.objectStore
    if not common_config:
        print("The clowder configuration doesn't provide object buckets configuration")
        return

    prefix = "https://" if common_config.tls else "http://"

    if bucket_name in buckets:
        bucket_config = buckets[bucket_name]
        configuration["access_key"] = bucket_config.accessKey
        configuration["secret_key"] = bucket_config.secretKey
        configuration["endpoint"] = f"{prefix}{common_config.hostname}:{common_config.port}"
    else:
        print("The bucket %s wasn't found among the Clowder buckets", bucket_name)


def _add_buckets_config(config):
    # Handle engine config
    engine = config.get("service", {}).get("engine", {})
    if engine.get("name") == "ccx_messaging.engines.s3_upload_engine.S3UploadEngine":
        engine_config = engine.get("kwargs", {})
        _update_bucket_config(engine_config.get("dest_bucket"), engine_config)

    # Handle downloader config
    downloader = config.get("service", {}).get("downloader", {})
    if downloader.get("name") == "ccx_messaging.downloaders.s3_downloader.S3Downloader":
        s3downloader_config = downloader.get("kwargs", {})
        _update_bucket_config(s3downloader_config.get("bucket"), s3downloader_config)



# Copyright 2024 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module that defines an Engine class for selecting where to publish a received archive."""

import logging
import tarfile

from ccx_messaging.error import CCXMessagingError

from insights.core.dr import Broker
from insights.formats import Formatter
from insights_messaging.engine import Engine


class MultiplexorEngine(Engine):
    """Engine that will create a report with the identified content of the archive."""

    def __init__(
        self,
        formatter: Formatter,
        target_components: list,
        extract_timeout: int | None = None,
        unpacked_archive_size_limit: int | None = None,
        extract_tmp_dir: str | None = None,
        filters: dict[str, str] | None = None,
    ):
        """Initialise the engine with the given filters.

        The `filter` argument is a dictionary where the keys are a path and the value, a mark.
        If a file with the given path is present inside the archive, it will be noted with
        the given "mark". An archive could have several matching files, so it could get a bunch
        of different marks.
        """
        super().__init__(
            formatter=formatter,
            target_components=target_components,
            extract_timeout=extract_timeout,
            unpacked_archive_size_limit=unpacked_archive_size_limit,
            extract_tmp_dir=extract_tmp_dir,
        )
        self.filters = filters if filters is not None else {}
        self.logger = logging.getLogger(__name__)

    def process(self, broker: Broker, path: str) -> set[str]:
        """Open an archive to check its content and classify it according to filters."""
        self.logger.info("Opening %s for multiplexing", path)
        self.fire("pre_extract", broker, path)

        try:
            with tarfile.open(path) as tf:
                self.fire("on_extract", None, broker, tf)

                filenames = tf.getnames()

                marks = set()
                for file_to_find, mark in self.filters.items():
                    if file_to_find in filenames:
                        marks.add(mark)

            if not marks:
                marks.add("DEFAULT")

        except tarfile.ReadError as ex:
            raise CCXMessagingError(
                "ReadError reading the archive", additional_data={"archive_path": path}
            ) from ex

        self.logger.info("Reporting marks: %s", marks)
        self.fire("on_engine_success", broker, marks)
        return marks



# Copyright 2024 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module for testing the engines module."""

import pytest
from insights.formats.text import HumanReadableFormat

from ccx_messaging.engines.multiplexor_engine import MultiplexorEngine


def test_init():
    """Test the MultiplexorEngine initializer."""
    sut = MultiplexorEngine(HumanReadableFormat, [], None, 10)

    # just basic check
    assert isinstance(sut, MultiplexorEngine)


def test_process_no_extract():
    """Basic test for MultiplexorEngine."""
    sut = MultiplexorEngine(HumanReadableFormat, [], None, 10)
    sut.watchers = []
    sut.extract_tmp_dir = "not-exist"

    broker = None
    path = "not-exist"

    with pytest.raises(Exception):
        sut.process(broker, path)


def test_process_no_filters():
    """Basic test for MultiplexorEngine."""
    sut = MultiplexorEngine(HumanReadableFormat, [], None, 10)
    sut.watchers = []
    sut.extract_tmp_dir = ""

    broker = None
    path = "test/wrong_data.tar"  # this file should be marked as "DEFAULT"

    result = sut.process(broker, path)
    assert result == {"DEFAULT"}


def test_process_filters_no_match():
    """Basic test for MultiplexorEngine."""
    filters = {
        "non-existing-file": "SHOULDNT MARK",
    }
    sut = MultiplexorEngine(HumanReadableFormat, [], None, 10, filters=filters)
    sut.watchers = []
    sut.extract_tmp_dir = ""

    broker = None
    path = "test/wrong_data.tar"  # this file should be marked as "DEFAULT"

    result = sut.process(broker, path)
    assert result == {"DEFAULT"}


def test_process_filters_match():
    """Basic test for MultiplexorEngine."""
    expected_mark = "WORKLOAD_INFO"

    filters = {
        "data/config/workload_info.json": expected_mark,
    }
    sut = MultiplexorEngine(HumanReadableFormat, [], None, 10, filters=filters)
    sut.watchers = []
    sut.extract_tmp_dir = ""

    broker = None
    path = "test/wrong_data.tar"  # this file should be marked as "DEFAULT"

    result = sut.process(broker, path)
    assert result == {expected_mark}


def test_process_several_matches():
    """Basic test for SHAExtractorEngine."""
    filters = {
        "config": "IO",
        "config/workload_info.json": "WORKLOAD_INFO",
    }

    sut = MultiplexorEngine(HumanReadableFormat, [], None, 10, filters=filters)
    sut.watchers = []
    sut.extract_tmp_dir = ""

    broker = None
    path = "test/correct_data.tar"

    result = sut.process(broker, path)
    assert "IO" in result
    assert "WORKLOAD_INFO" in result
    assert len(result) == 2


def test_multiplexor_engine_tarfile_error():
    """Test MultiplexorEngine tarfile.ReadError handling."""
    import tarfile
    from unittest.mock import patch
    from ccx_messaging.error import CCXMessagingError

    sut = MultiplexorEngine(HumanReadableFormat, [], None, 10)
    sut.watchers = []
    sut.extract_tmp_dir = ""

    broker = None
    test_path = "/tmp/test_corrupted_archive.tar"

    # Mock tarfile.open to raise ReadError
    with patch("tarfile.open") as mock_open:
        mock_open.side_effect = tarfile.ReadError("file could not be opened successfully")

        # Test that CCXMessagingError is raised with correct format
        with pytest.raises(CCXMessagingError) as exc_info:
            sut.process(broker, test_path)

        # Verify the error structure
        error = exc_info.value
        assert str(error) == "ReadError reading the archive"
        assert error.additional_data is not None
        assert error.additional_data["archive_path"] == test_path



# Copyright 2020 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing unit tests for the `ConsumerWatcher` class."""

from unittest.mock import MagicMock, patch

import pytest
from freezegun import freeze_time

from ccx_messaging.watchers.payload_tracker_watcher import PayloadTrackerWatcher


_INVALID_SERVERS = [
    None,
    [],
    {},
    100,
    100.5,
    "kafka_instance",
]

_INVALID_TOPICS = [
    None,
    [],
    {},
    "",
]


def _prepare_kafka_mock(producer_init_mock):
    """Create a producer mock from its mocked initialization."""
    producer_mock = MagicMock()
    producer_init_mock.return_value = producer_mock
    return producer_mock


@pytest.mark.parametrize("bootstrap_value", _INVALID_SERVERS)
@pytest.mark.parametrize("topic_value", _INVALID_TOPICS)
def test_payload_tracker_watcher_invalid_initialize_invalid_servers(bootstrap_value, topic_value):
    """Test passing invalid data types or values to the `PayloadTrackerWatcher` initializer."""
    with pytest.raises((TypeError, PermissionError, OverflowError, KeyError)):
        _ = PayloadTrackerWatcher(topic_value, **{"bootstrap.servers": bootstrap_value})


@patch("ccx_messaging.watchers.payload_tracker_watcher.Producer")
def test_payload_tracker_init_with_kafka_config(producer_init_mock):
    """Check that passing a kafka_broker_config parameter updates the default ones."""
    kafka_broker_cfg = {"bootstrap.servers": "valid_server"}

    PayloadTrackerWatcher(
        "valid_topic",
        kafka_broker_config=kafka_broker_cfg,
        **{"bootstrap.servers": "invalid_servicer"},
    )

    producer_init_mock.assert_called_with(**kafka_broker_cfg)


@freeze_time("2020-05-07T14:00:00")
@patch("ccx_messaging.watchers.payload_tracker_watcher.Producer")
def test_payload_tracker_watcher_publish_status(producer_init_mock):
    """Test publish_status method sends the expected value to Kafka."""
    mocked_values = {
        "request_id": "some request id",
        "identity": {
            "identity": {
                "internal": {
                    "org_id": 1,
                },
                "account_number": 2,
            }
        },
    }

    producer_mock = _prepare_kafka_mock(producer_init_mock)
    sut = PayloadTrackerWatcher("valid_topic", **{"bootstrap.servers": "bootstrap_server"})
    sut.on_recv(mocked_values)
    producer_mock.produce.assert_called_with(
        "valid_topic",
        b'{"service": "ccx-data-pipeline", "request_id": "some request id", '
        b'"status": "received", "date": "2020-05-07T14:00:00", '
        b'"org_id": 1, "account": 2}',
    )

    sut.on_process(mocked_values, "{result}")
    producer_mock.produce.assert_called_with(
        "valid_topic",
        b'{"service": "ccx-data-pipeline", "request_id": "some request id", '
        b'"status": "processing", "date": "2020-05-07T14:00:00", '
        b'"org_id": 1, "account": 2}',
    )

    sut.on_consumer_success(mocked_values, "broker", "{result}")
    producer_mock.produce.assert_called_with(
        "valid_topic",
        b'{"service": "ccx-data-pipeline", "request_id": "some request id", '
        b'"status": "success", "date": "2020-05-07T14:00:00", '
        b'"org_id": 1, "account": 2}',
    )

    sut.on_consumer_failure(mocked_values, Exception("Something"))
    producer_mock.produce.assert_called_with(
        "valid_topic",
        b'{"service": "ccx-data-pipeline", "request_id": "some request id", '
        b'"status": "error", "date": "2020-05-07T14:00:00", '
        b'"org_id": 1, "account": 2, "status_msg": "Something"}',
    )

    # call _publish_status without request_id and check there is not any
    # exception thrown
    del mocked_values["request_id"]
    sut.on_recv(mocked_values)



# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module that defines a Downloader object to get HTTP urls."""

import logging
import re
from contextlib import contextmanager
from tempfile import NamedTemporaryFile

import requests

from ccx_messaging.error import CCXMessagingError

LOG = logging.getLogger(__name__)


def parse_human_input(file_size):
    """Parse an input in human-readable format and return a number of bytes."""
    multipliers = {
        "K": 10**3,
        "M": 10**6,
        "G": 10**9,
        "T": 10**12,
        "Ki": 2**10,
        "Mi": 2**20,
        "Gi": 2**30,
        "Ti": 2**40,
    }

    match = re.match(r"^(?P<quantity>\d+(\.\d+)?)\s*(?P<units>[KMGT]?i?B?)?$", file_size)

    if match is None:
        raise ValueError(f"The file size cannot be parsed as a file size: {file_size}")

    parsed = match.groupdict()
    quantity = float(parsed.get("quantity"))

    units = parsed.get("units")
    units = units.rstrip("B") if units is not None else ""

    if units != "" and units not in multipliers:
        raise ValueError(f"The file size cannot be parsed because its units: {parsed.get('units')}")

    multiplier = multipliers.get(units, 1)  # if multiplier == "", then 1
    quantity = quantity * multiplier
    return int(quantity)


# pylint: disable=too-few-public-methods
class HTTPDownloader:
    """Downloader for HTTP uris."""

    # https://<hostname>/service_id/file_id?<credentials and other params>
    HTTP_RE = re.compile(
        r"^(?:https://[^/]+\.s3\.amazonaws\.com/[0-9a-zA-Z/\-]+|"
        r"https://s3\.[0-9a-zA-Z\-]+\.amazonaws\.com/[0-9a-zA-Z\-]+/[0-9a-zA-Z/\-]+)\?"
        r"X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=[^/]+$"
    )

    def __init__(self, max_archive_size=None, allow_unsafe_links=False):
        """`HTTPDownloader` initializer.

        This method accepts a `max_archive_size` argument, that indicates the
        maximum size allowed for the archives. If set, archives bigger than this
        will be discarded.
        """
        if max_archive_size is not None:
            self.max_archive_size = parse_human_input(max_archive_size)
            LOG.info("Configured max_archive_size to %s bytes", self.max_archive_size)

        else:
            self.max_archive_size = None
            LOG.warning("No max_archive_size defined. Be careful")

        self.allow_unsafe_links = allow_unsafe_links

    @contextmanager
    def get(self, src):
        """Download a file from HTTP server and store it in a temporary file."""
        if not self.allow_unsafe_links:
            if src is None or not HTTPDownloader.HTTP_RE.fullmatch(src):
                LOG.warning("Invalid URL format: %s", src)
                raise CCXMessagingError("Invalid URL format")

        try:
            response = requests.get(src)
            data = response.content
            size = len(data)

            if size == 0:
                LOG.warning("Empty input archive from: %s", src)
                raise CCXMessagingError("Empty input archive")

            if self.max_archive_size is not None and size > self.max_archive_size:
                LOG.warning(
                    "The archive is too big (%d > %d). Skipping", size, self.max_archive_size
                )
                raise CCXMessagingError("The archive is too big. Skipping")

            with NamedTemporaryFile() as file_data:
                file_data.write(data)
                file_data.flush()
                yield file_data.name
            response.close()

        except requests.exceptions.ConnectionError as err:
            LOG.warning("Connection error while downloading the file: %s", err)
            raise CCXMessagingError("Connection error while downloading the file") from err

        except CCXMessagingError as err:
            additional_data = getattr(err, "additional_data", None)
            if additional_data is not None:
                LOG.warning(
                    "Error while downloading the file: %s",
                    err,
                    extra=additional_data,
                )
            else:
                LOG.warning("Error while downloading the file: %s", err)

            raise err

        except Exception as err:
            LOG.warning("Unknown error while downloading the file: %s", err)
            raise CCXMessagingError("Unknown error while downloading the file") from err



"""Kafka consumer implementation for multiplexor messages using Confluent Kafka library."""

import json
import logging

import jsonschema
from confluent_kafka import Message

from ccx_messaging.consumers.kafka_consumer import KafkaConsumer
from ccx_messaging.error import CCXMessagingError
from ccx_messaging.schemas import IDENTITY_SCHEMA

LOG = logging.getLogger(__name__)


class DecodedIngressConsumer(KafkaConsumer):
    """Kafka consumer for decoded ingress messages produced by KafkaConsumer."""

    INPUT_MESSAGE_SCHEMA = {
        "type": "object",
        "properties": {
            "url": {"type": "string"},
            "identity": IDENTITY_SCHEMA,
            "timestamp": {"type": "string"},
            "cluster_name": {"type": ["string", "null"]},
        },
        "required": ["url", "identity", "timestamp"],
    }

    @staticmethod
    def parse_decoded_ingress_message(message: bytes) -> dict:
        """Parse a bytes messages into a dictionary, decoding encoded values."""
        try:
            deserialized_message = json.loads(message)
            jsonschema.validate(
                instance=deserialized_message, schema=DecodedIngressConsumer.INPUT_MESSAGE_SCHEMA
            )

        except TypeError as ex:
            LOG.warning("Incorrect message type: %s", message)
            raise CCXMessagingError("Incorrect message type") from ex

        except json.JSONDecodeError as ex:
            LOG.warning("Unable to decode received message: %s", message)
            raise CCXMessagingError("Unable to decode received message") from ex

        except jsonschema.ValidationError as ex:
            LOG.warning("Invalid input message JSON schema: %s", deserialized_message)
            raise CCXMessagingError("Invalid input message JSON schema") from ex

        LOG.debug("JSON schema validated: %s", deserialized_message)

        return deserialized_message

    def deserialize(self, msg: Message) -> dict:
        """Deserialize the message received from Kafka into a dictionary."""
        if not msg:
            raise CCXMessagingError("No incoming message: %s", msg)

        try:
            value = msg.value()
        except AttributeError as ex:
            raise CCXMessagingError("Invalid incoming message type: %s", type(msg)) from ex

        LOG.debug("Deserializing incoming message(%s): %s", self.log_pattern, value)

        if not value:
            raise CCXMessagingError("Unable to read incoming message: %s", value)

        deserialized_msg = self.parse_decoded_ingress_message(value)
        LOG.debug("JSON message deserialized (%s): %s", self.log_pattern, deserialized_msg)

        if not deserialized_msg.get("cluster_name"):
            cluster_id = (
                deserialized_msg.get("identity", {})
                .get("identity", {})
                .get("system", {})
                .get("cluster_id", None)
            )
            deserialized_msg["cluster_name"] = cluster_id

        return deserialized_msg



# Copyright 2024 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module for testing the engines module."""

from insights.formats.text import HumanReadableFormat
import pytest

from ccx_messaging.engines.ocp_engine import OCPEngine
from ccx_messaging.watchers.stats_watcher import StatsWatcher


def test_init():
    """Test the OCPEngine constructor."""
    e = OCPEngine(HumanReadableFormat)

    # just basic check
    assert e is not None


def test_process_no_extract():
    """Basic test for OCPEngine."""
    e = OCPEngine(HumanReadableFormat)
    e.watchers = []
    e.extract_tmp_dir = "not-exist"

    broker = None
    path = "not-exist"

    with pytest.raises(Exception):
        e.process(broker, path)


def test_process_extract_wrong_data():
    """Basic test for OCPEngine."""
    e = OCPEngine(HumanReadableFormat)
    e.watchers = []
    e.extract_tmp_dir = ""

    broker = None
    path = "test/wrong_data.tar"

    result = e.process(broker, path)
    assert result is not None


def test_process_extract_correct_data():
    """Basic test for OCPEngine."""
    e = OCPEngine(HumanReadableFormat)
    e.watchers = []
    e.extract_tmp_dir = ""

    broker = None
    path = "test/correct_data.tar"

    result = e.process(broker, path)
    assert result is not None


def test_process_extract_ols_archive():
    """Basic test for OCPEngine with an OLS file."""
    e = OCPEngine(HumanReadableFormat)
    e.watchers = []
    e.extract_tmp_dir = ""

    broker = None
    path = "test/ols.tar"

    result = e.process(broker, path)
    assert result == "{}"


def test_process_extract_rapid_recommendation_archive():
    """Basic test for OCPEngine with a rapid recommendation file."""
    e = OCPEngine(HumanReadableFormat)
    sw = StatsWatcher()
    e.watchers = [sw]
    e.extract_tmp_dir = ""

    broker = None
    path = "test/rapid-recommendations.tar.gz"

    result = e.process(broker, path)
    assert result != "{}"
    assert sw._gathering_conditions_remote_configuration_version.labels("1.1.0")._value.get() == 1



# Copyright 2023 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing unit tests for the `Consumer` class."""

import sys
import pytest


import ccx_messaging.command_line as command_line


def test_command_line_args_valid_flag_version():
    """Verify correct parsing of --version flag."""
    sys.argv = ["ccx-messaging", "--version"]
    parser = command_line.parse_args()
    assert not parser.config
    assert parser.version


def test_command_line_args_valid_config_value():
    """Verify correct parsing of positional argument <config>."""
    sys.argv = ["ccx-messaging", "path_to_config_file"]
    parser = command_line.parse_args()
    assert not parser.version
    assert parser.config


def test_command_line_args_no_config_provided():
    """Verify app does not start if no config provided and exit with code 1."""
    with pytest.raises(SystemExit) as exception:
        sys.argv = ["ccx-messaging"]
        command_line.ccx_messaging()

    assert exception.type is SystemExit
    assert exception.value.code == 1


def test_command_line_args_invalid_arg_provided(capsys):
    """Verify help is shown if invalid argument provided."""
    sys.argv = ["ccx-messaging", "--config"]
    with pytest.raises(SystemExit) as exception:
        parser = command_line.parse_args()

        assert exception.type is SystemExit
        assert exception.value.code == 2

        assert not parser.config
        assert not parser.version

        captured = capsys.readouterr()
        assert "usage: ccx-messaging" in captured.out
        assert "error: unrecognized arguments" in captured.out



# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing a `Watcher` to send updates to the `payload-tracker` service."""

import datetime
import json
import logging

from confluent_kafka import Producer

from ccx_messaging.utils.kafka_config import kafka_producer_config_cleanup
from ccx_messaging.watchers.consumer_watcher import ConsumerWatcher


LOG = logging.getLogger(__name__)


class PayloadTrackerWatcher(ConsumerWatcher):
    """`Watcher` implementation to handle Payload Tracker updates."""

    def __init__(self, topic, service_name="ccx-data-pipeline", kafka_broker_config=None, **kwargs):
        """Construct a `PayloadTrackerWatcher` object."""
        self.topic = topic

        if not self.topic:
            raise KeyError("topic")

        if kafka_broker_config:
            kwargs.update(kafka_broker_config)

        kwargs = kafka_producer_config_cleanup(kwargs)

        LOG.info(
            "Confluent Kafka consumer configuration arguments: "
            "Server: %s. "
            "Topic: %s. "
            "Security protocol: %s.",
            kwargs.get("bootstrap.servers"),
            self.topic,
            kwargs.get("security.protocol"),
        )
        self.kafka_prod = Producer(**kwargs)
        self.service_name = service_name

        LOG.info(
            "Sending status reports to Payload Tracker on topic %s as service %s",
            self.topic,
            self.service_name,
        )

    def _publish_status(self, input_msg, status, status_msg=None):
        """Send an status update to payload tracker topic."""
        request_id = input_msg.get("request_id")
        identity_field = input_msg.get("identity", {}).get("identity", {})
        org_id = identity_field.get("internal", {}).get("org_id")
        account = identity_field.get("account_number")
        cluster_name = input_msg.get("cluster_name", "")

        if request_id is None:
            LOG.warning("The received record doesn't contain a request_id. It won't be reported")
            return

        tracker_msg = {
            "service": self.service_name,
            "request_id": request_id,
            "status": status,
            "date": datetime.datetime.now().isoformat(),
        }

        if org_id:
            tracker_msg["org_id"] = org_id

        if account:
            tracker_msg["account"] = account

        if status_msg:
            tracker_msg["status_msg"] = status_msg

        self.kafka_prod.produce(self.topic, json.dumps(tracker_msg).encode("utf-8"))
        self.kafka_prod.poll(0)
        LOG.debug(
            "Payload Tracker update successfully sent for cluster %s: %s %s",
            cluster_name,
            request_id,
            status,
        )

    def on_recv(self, input_msg):
        """On received event handler."""
        self._publish_status(input_msg, "received")

    def on_process(self, input_msg, results):
        """On processing a new archive event handler."""
        self._publish_status(input_msg, "processing")

    def on_consumer_success(self, input_msg, broker, results):
        """On consumer success event handler."""
        self._publish_status(input_msg, "success")

    def on_consumer_failure(self, input_msg, exception):
        """On consumer failure event handler."""
        self._publish_status(input_msg, "error", str(exception))



# Copyright 2024 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing unit tests for the `KafkaConsumer` class."""

import datetime
from unittest.mock import MagicMock, patch

import pytest
from freezegun import freeze_time

from ccx_messaging.consumers.idp_kafka_consumer import IDPConsumer
from ccx_messaging.error import CCXMessagingError

from . import KafkaMessage


# _REGEX_BAD_SCHEMA = r"^Unable to extract URL from input message: "
_INVALID_TYPE_VALUES = [
    None,
    42,
    3.14,
    True,
    [],
    {},
]


@pytest.mark.parametrize("value", _INVALID_TYPE_VALUES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer")
def test_deserialize_invalid_type(mock_consumer, value):
    """Test that passing invalid data type to `deserialize` raises an exception."""
    sut = IDPConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(value)


@pytest.mark.parametrize("value", _INVALID_TYPE_VALUES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer")
def test_deserialize_invalid_type_as_kafka_message(mock_consumer, value):
    """Test that passing invalid data type to `deserialize` raises an exception."""
    sut = IDPConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(KafkaMessage(value))


_VALID_MESSAGES = [
    (
        '{"path": ""}',
        {
            "path": "",
        },
    ),
    (
        '{"path": "","cluster_id": "3d59cd30-e2c3-4ec8-b35d-7df3b1feea2f"}',
        {
            "path": "",
            "cluster_id": "3d59cd30-e2c3-4ec8-b35d-7df3b1feea2f",
        },
    ),
    (
        '{"path": "",'
        '"cluster_id": "3d59cd30-e2c3-4ec8-b35d-7df3b1feea2f",'
        '"sqs_message_id": "a_very_long_sqs_id"'
        "}",
        {
            "path": "",
            "cluster_id": "3d59cd30-e2c3-4ec8-b35d-7df3b1feea2f",
            "sqs_message_id": "a_very_long_sqs_id",
        },
    ),
    (
        '{"path": "path/inside/bucket/file.tgz",'
        '"cluster_id": "3d59cd30-e2c3-4ec8-b35d-7df3b1feea2f",'
        '"sqs_message_id": "a_very_long_sqs_id"'
        "}",
        {
            "path": "path/inside/bucket/file.tgz",
            "cluster_id": "3d59cd30-e2c3-4ec8-b35d-7df3b1feea2f",
            "sqs_message_id": "a_very_long_sqs_id",
        },
    ),
]


@pytest.mark.parametrize("msg,value", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_valid_str(msg, value):
    """Test that proper string JSON input messages are correctly deserialized."""
    sut = IDPConsumer(None, None, None, incoming_topic=None)
    assert sut.deserialize(KafkaMessage(msg)) == value


@pytest.mark.parametrize("msg,value", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_valid_bytes(msg, value):
    """Test that proper string JSON input messages are correctly deserialized."""
    sut = IDPConsumer(None, None, None, incoming_topic=None)
    assert sut.deserialize(KafkaMessage(msg.encode())) == value


@pytest.mark.parametrize("msg,value", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_valid_bytearray(msg, value):
    """Test that proper string JSON input messages are correctly deserialized."""
    sut = IDPConsumer(None, None, None, incoming_topic=None)
    assert sut.deserialize(KafkaMessage(bytearray(msg.encode()))) == value


_INVALID_MESSAGES = [
    "",
    '"path": "value"',
]


@pytest.mark.parametrize("msg", _INVALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_deserialize_invalid_str(msg):
    """Test that invalid string JSON is not correctly deserialized."""
    sut = IDPConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.deserialize(KafkaMessage(msg))


@pytest.mark.parametrize("msg, _", _VALID_MESSAGES)
@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_handles_valid(msg, _):
    """Test that `handles` method returns True for valid messages."""
    sut = IDPConsumer(None, None, None, incoming_topic=None)
    assert sut.handles(KafkaMessage(msg))


# This would have been a valid input, but it's supposed to be a `dict`, not `str`.
_DICT_STR = '{"path": "bucket/file"}'

_INVALID_RECORD_VALUES = [
    "",
    _DICT_STR.encode("utf-8"),
    bytearray(_DICT_STR.encode("utf-8")),
    [],
]

_VALID_RECORD_VALUES = [
    {"path": ""},
    {"path": "bucket/file"},
    {"path": "https://a-valid-domain.com/precious_url"},
]

_VALID_EMPTY_RECORD_VALUES = [
    {},
    {"notpath": "bucket/file"},
]


@pytest.mark.parametrize("value", _INVALID_RECORD_VALUES)
def test_get_url_invalid(value):
    """Test that `IDPConsumer.get_url` raises the appropriate exception."""
    with pytest.raises(Exception):
        IDPConsumer.get_url(None, value)


@pytest.mark.parametrize("value", _VALID_EMPTY_RECORD_VALUES)
def test_get_url_empty(value):
    """Test that `IDPConsumer.get_url` returns None."""
    assert IDPConsumer.get_url(None, value) is None


@pytest.mark.parametrize("value", _VALID_RECORD_VALUES)
def test_get_url_valid(value):
    """Test that `IDPConsumer.get_url` returns the expected value."""
    assert IDPConsumer.get_url(None, value) == value["path"]


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_create_broker():
    """Test that `create_broker` generates a broker with the expected values."""
    path = (
        "00000000/11111111-2222-3333-4444-555555555555/"
        "66666666666666-77777777777777777777777777777777"
    )
    input_msg = {
        "path": path,
    }

    sut = IDPConsumer(None, None, None, incoming_topic=None)
    broker = sut.create_broker(input_msg)

    assert broker["cluster_id"] == "11111111-2222-3333-4444-555555555555"
    assert broker["original_path"] == input_msg["path"]
    assert broker["org_id"] == "00000000"
    assert broker["year"] == "6666"
    assert broker["month"] == "66"
    assert broker["day"] == "66"
    assert broker["time"] == "666666"
    assert broker["id"] == "77777777777777777777777777777777"


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_create_broker_with_cluster_id_precedence():
    """Test that `create_broker` generates a broker with the expected values."""
    path = (
        "00000000/11111111-2222-3333-4444-555555555555/66666666666666-"
        "77777777777777777777777777777777"
    )
    input_msg = {
        "path": path,
        "cluster_id": "6a627474-05fc-41b1-8711-45ccc68238fe",
    }

    sut = IDPConsumer(None, None, None, incoming_topic=None)
    broker = sut.create_broker(input_msg)

    assert broker["cluster_id"] == input_msg["cluster_id"]
    assert broker["original_path"] == input_msg["path"]
    assert broker["org_id"] == "00000000"
    assert broker["year"] == "6666"
    assert broker["month"] == "66"
    assert broker["day"] == "66"
    assert broker["time"] == "666666"
    assert broker["id"] == "77777777777777777777777777777777"


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_create_broker_bad_path():
    """Test that `create_broker` raises an error when the path format doesn't match."""
    path = "11111111-2222-3333-4444-555555555555/66666666666666-77777777777777777777777777777777"

    input_msg = {
        "path": path,
        "cluster_id": "6a627474-05fc-41b1-8711-45ccc68238fe",
    }

    sut = IDPConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.create_broker(input_msg)


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_create_broker_bad_cluster_id_format():
    """Test that `create_broker` generates a broker with the expected values."""
    path = (
        "00000000/aaaaaaaa-bbbb-xxxx-ffff-badbadbadbad/"
        "66666666666666-77777777777777777777777777777777"
    )
    input_msg = {
        "path": path,
    }

    sut = IDPConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.create_broker(input_msg)


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
def test_create_broker_bad_cluster_id_short():
    """Test that `create_broker` generates a broker with the expected values."""
    path = "00000000/aa-bbbb-cc-ffff-badbadbadbad/66666666666666-77777777777777777777777777777777"
    input_msg = {
        "path": path,
    }

    sut = IDPConsumer(None, None, None, incoming_topic=None)
    with pytest.raises(CCXMessagingError):
        sut.create_broker(input_msg)


@patch("ccx_messaging.consumers.kafka_consumer.ConfluentConsumer", lambda *a, **k: MagicMock())
@patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.handles", lambda *a, **k: True)
@patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.fire", lambda *a, **k: None)
def test_last_received_message_time_is_updated():
    """Check that the variable last_received_message_time used by a Thread is correctly updated.

    [CCXDEV-14812] the variable is never updated
    """
    t1 = datetime.datetime(2025, 1, 31, 12, 0, 0, tzinfo=datetime.timezone.utc)
    t2 = datetime.datetime(2025, 1, 31, 12, 20, 0, tzinfo=datetime.timezone.utc)

    with freeze_time(t1) as frozen_time:
        sut = IDPConsumer(None, None, None, incoming_topic=None)
        input_msg = KafkaMessage("{}")

        frozen_time.move_to(t2)

        with patch("ccx_messaging.consumers.kafka_consumer.KafkaConsumer.process", lambda: None):
            sut.process_msg(input_msg)

        assert sut.last_received_message_time == t2.timestamp()



# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing the implementation of the `CCXMessagingError` exception class."""


class CCXMessagingError(Exception):
    """Represents a CCX messaging exception.

    This should make it easier to differentiate between
    exceptions caused by internal and external code.
    """

    def __init__(self, message, *args, additional_data=None):
        """Initialize CCXMessagingError with optional additional data.

        Args:
            message: The error message (may contain format specifiers)
            *args: Arguments for string formatting (for backward compatibility)
            additional_data: Optional dict containing additional context data

        Raises:
            TypeError: If additional_data is not None or dict

        """
        super().__init__(message, *args)

        if additional_data is not None and not isinstance(additional_data, dict):
            raise TypeError("additional_data must be a dict or None")
        self.additional_data = additional_data

    def format(self, input_msg):
        """Format the error by adding information about input Kafka message."""
        return f"Status: Error; Topic: {input_msg['topic']}; Cause: {self}"



"""Kafka consumer implementation using Confluent Kafka library for Internal Data Pipeline."""

import logging
import json
import re

from confluent_kafka import Message

from ccx_messaging.consumers.kafka_consumer import KafkaConsumer
from ccx_messaging.error import CCXMessagingError
from ccx_messaging.monitored_broker import SentryMonitoredBroker


# Path example: <org_id>/<cluster_id>/<year><month><day><time>-<id>
# Following RE matches with S3 archives like the previous example and allow
# to extract named groups for the different meaningful components
S3_ARCHIVE_PATTERN = re.compile(
    r"(?P<org_id>[0-9]+)\/"  # extract named group for organization id
    r"(?P<cluster_id>[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12})\/"  # extract named group for the cluster_id  # noqa: E501
    r"(?P<archive>"  # extract named group for the archive name, including the following 3 lines
    r"(?P<timestamp>"  # extract the timestamp named group, including the following line
    # Next line extract year, month, day and time named groups from the timestamp
    r"(?P<year>[0-9]{4})(?P<month>[0-9]{2})(?P<day>[0-9]{2})(?P<time>[0-9]{6}))-"
    r"(?P<id>[a-z,A-Z,0-9]*))"  # Extract the id of the file as named group
)
LOG = logging.getLogger(__name__)


class IDPConsumer(KafkaConsumer):
    """Consumer based in Confluent Kafka for Internal Data Pipeline."""

    def __init__(
        self,
        publisher,
        downloader,
        engine,
        **kwargs,
    ):
        """Initialise the KafkaConsumer object and related handlers."""
        kwargs.pop("requeuer", None)
        incoming_topic = kwargs.pop("incoming_topic")
        super().__init__(publisher, downloader, engine, incoming_topic, **kwargs)

    def get_url(self, input_msg: dict) -> str:
        """Retrieve URL to storage from Kafka message."""
        return input_msg.get("path")

    def handles(self, msg: Message) -> bool:
        """Check if the message is usable."""
        message = self.deserialize(msg)
        return "path" in message

    def deserialize(self, msg):
        """Deserialize JSON message received from kafka."""
        if not msg:
            raise CCXMessagingError("No incoming message %s", msg)

        try:
            value = msg.value()

        except AttributeError as ex:
            raise CCXMessagingError("Invalid incoming message type: %s", type(msg)) from ex

        LOG.debug("Deserializing incoming message(%s): %s", self.log_pattern, value)

        try:
            deserialized_message = json.loads(value)

        except TypeError as ex:
            LOG.warning("Incorrect message type: %s", msg)
            raise CCXMessagingError("Incorrect message type") from ex

        except json.JSONDecodeError as ex:
            LOG.warning("Unable to decode received message: %s", msg)
            raise CCXMessagingError("Unable to decode received message") from ex

        LOG.debug("JSON schema validated: %s", deserialized_message)
        return deserialized_message

    def create_broker(self, input_msg):
        """Create a suitable `Broker` to be pass arguments to the `Engine`."""
        path = input_msg.get("path")
        broker = SentryMonitoredBroker()
        broker["original_path"] = path

        if "cluster_id" in input_msg:
            broker["cluster_id"] = input_msg["cluster_id"]

        match_ = S3_ARCHIVE_PATTERN.match(path)
        if not match_:
            LOG.warning("The archive doesn't match the expected pattern: %s", path)
            exception = CCXMessagingError("Archive pattern name incorrect")
            self.fire("on_consumer_failure", broker, exception)
            raise exception

        # Cluster ID might be overrided by the one found in the `path`
        for key, value in match_.groupdict().items():
            if key not in broker:
                broker[key] = value

        return broker



"""Functions to adapt kafka-python configuration parameters."""


def translate_kafka_configuration(config: dict) -> dict:
    """Transform a dict with default Kafka configuration to kafka-python configuration."""
    lib_config = {}

    if not config:
        return {}

    keys_translation = {
        "bootstrap.servers": "bootstrap_servers",
        "ssl.ca.location": "ssl_cafile",
        "sasl.mechanisms": "sasl_mechanism",
        "sasl.username": "sasl_plain_username",
        "sasl.password": "sasl_plain_password",
        "security.protocol": "security_protocol",
    }

    for kafka_key, lib_key in keys_translation.items():
        if kafka_key not in config:
            continue

        lib_config[lib_key] = config[kafka_key]

    return lib_config


def kafka_producer_config_cleanup(config):
    """Clean up the configuration dictionary of consumer-only properties."""
    consumer_only_properties = [
        "group.id",
        "session.timeout.ms",
        "heartbeat.interval.ms",
        "max.poll.interval.ms",
    ]

    for property in consumer_only_properties:
        if property in config:
            del config[property]

    return config



# Copyright 2020 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Module containing unit tests for the `ConsumerWatcher` class."""

from unittest.mock import MagicMock, mock_open, patch

import pytest

from ccx_messaging.watchers.cluster_id_watcher import ClusterIdWatcher


def test_cluster_id_watcher_unordered_events(caplog):
    """Test that an extraction event is received without a receiving one ."""
    sut = ClusterIdWatcher()
    sut.on_extract(None, None, None)

    assert sut.last_record is None
    assert "Unexpected data flow" in caplog.text


def test_cluster_id_watcher_file_not_exist(caplog):
    """Test that the extracted archive doesn't contain a cluster id file."""
    input_msg = {"identity": {}}

    extraction_mock = MagicMock()
    extraction_mock.tmp_dir = "/tmp/to/non/existing/file/over/the/filesystem"

    sut = ClusterIdWatcher()
    sut.on_recv(input_msg)

    sut.on_extract(None, None, extraction_mock)

    assert input_msg["cluster_name"] is None
    assert "The archive doesn't contain a valid Cluster Id file" in caplog.text


def test_cluster_id_watcher_io_error(caplog):
    """Test the behaviour in case of I/O error."""
    input_msg = {"identity": {}}

    extraction_mock = MagicMock()
    # this is a trick mentioned there
    # https://unix.stackexchange.com/questions/6301/how-do-i-read-from-proc-pid-mem-under-linux/
    extraction_mock.tmp_dir = "/proc/self/mem"

    sut = ClusterIdWatcher()
    sut.on_recv(input_msg)

    sut.on_extract(None, None, extraction_mock)
    assert input_msg["cluster_name"] is None
    assert "Could not read file: " in caplog.text


_INCORRECT_UUIDS = [
    "0",
    "---//---",
    "hhhh-bbbb-cccc-dddd-000000000000",
]


@pytest.mark.parametrize("value", _INCORRECT_UUIDS)
def test_cluster_id_watcher_bad_content(caplog, value):
    """Test that the id file contains non UUID values."""
    input_msg = {"identity": {}}

    extraction_mock = MagicMock()
    extraction_mock.tmp_dir = "/tmp/mock/path"

    sut = ClusterIdWatcher()
    sut.on_recv(input_msg)

    with patch("builtins.open", mock_open(read_data=value)):
        sut.on_extract(None, None, extraction_mock)
        assert input_msg["cluster_name"] is None
        assert "The cluster id is not an UUID" in caplog.text


def test_cluster_id_watcher_ok(caplog):
    """Test that the id file contains non UUID values."""
    input_msg = {"identity": {}}

    extraction_mock = MagicMock()
    extraction_mock.tmp_dir = "/tmp/mock/path"

    sut = ClusterIdWatcher()
    sut.on_recv(input_msg)

    uuid_value = "aaaaaaaa-bbbb-cccc-dddd-000000000000"

    with patch("builtins.open", mock_open(read_data=uuid_value)):
        sut.on_extract(None, None, extraction_mock)
        assert input_msg["cluster_name"] == uuid_value
        assert len(caplog.records) == 0

        sut.last_record = MagicMock()
        sut.last_record = {"cluster_name": uuid_value}

        # now the cluster name should be set already
        # so the method ends w/o opening the file
        sut.on_extract(None, None, extraction_mock)
        assert input_msg["cluster_name"] == uuid_value
